<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015-2016 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 
namespace core;

if(!defined('RS-MINI')) die();

/*
 * helper v 1.0
 * 
 * класс обвязка для вспомогательных классов инструментария
 * написан для того что бы хранить методы в разных классах
 * а вызывать из одного
 * 
 * два варианта вызова
 *  - helper::методИнструментария(); - только в том 
 *                                     случае если метод
 *                                     имеет уникальное
 *                                     имя
 *  - helper::get('псевдоним инструментария')
 *            ->методИнструментария()
 *    ;
 *
 */

class helper
{
    /*
     * объекты инструментария (синглтоны)
     *
     * @var    - array
     * @access - private
     */
    
    private static $objectList = NULL;
    
    /*
     * список инструментария (псевдонимы (ключи)
     * и имена классов)
     *
     * @var    - array
     * @access - private
     */
    
    private static $aliasList = NULL;
    
    /*
     * по имени метода определяем объект класса в
     * котором есть вызываемый метод после чего
     * вызываем вызываемый метод из найденного объекта
     * 
     * внимание. пользоваться только в том случае если
     * метод имеет уникальное имя. в противном случае будет
     * вызван метод из первого найденного объекта. в случае если
     * уникальность не соблюдалась, можно воспользоваться методом
     * get и принудительно указать псевдоним (задавался при
     * регистрации инструментария) класса инструментария из которого
     * нужно вызвать метод
     * 
     * @access - public
     * @return - результат отработки вызываемого метода или die
     *
     * @param string name         - имя вызываемого метода
     * @param array  argumentList - список аргументов
     */
    
    public static function __callStatic($name, $argumentList)
    {
        // если нет списка объектов
        if (is_null(self::$aliasList))
            // выводим еррор
            self::getError('method ' . $name . ' is not found');
            
        // листаем инструменты
        foreach (self::$aliasList as $alias => $class)
            // если вызываемый метод существует в данном объекте
            if (method_exists($class, $name)) {
                // проверяем существует ли объект в списке уже
                // поднятых объектах
                if (!isset(self::$objectList[$alias]))
                    // если нет, поднимаем объект
                    self::$objectList[$alias] = new self::$aliasList[$alias]();
                
                // запускаем метод
                return call_user_func_array(
                    array(
                        // объект
                        self::$objectList[$alias],
                        // метод объекта
                        $name
                    ),
                    // список передаваемых параметров
                    $argumentList
                );
            }
        
        // убиваем скрипт
        self::getError('method ' . $name . ' is not found');
    }
    
    /*
     * метод выбирает объект по алиасу (если такого
     * объекта нет, поднимает его)
     * 
     * @access - public
     * 
     * @param string alias - псевдоним хелпера
     */
    
    public static function get($alias)
    {
        // если объект существует в списке уже
        // поднятых объектах
        if (isset(self::$objectList[$alias]))
            // вертаем сслыку на этот объект
            return self::$objectList[$alias];
            
        // если существует только имя класса
        if (isset(self::$aliasList[$alias]))
            // поднимаем объект сохраняя его в списке
            return self::$objectList[$alias] = new self::$aliasList[$alias]();
        
        // если система ничего не знает про существование
        // запрашиваемого класса, то вертаем эррор
        self::getError('class ' . $alias . ' is not found');
    }
    
    /*
     * метод поднимает отдельный (независимый)
     * объект класса
     * 
     * @access - public
     * 
     * @param string alias - псевдоним хелпера
     */
    
    public static function create($alias)
    {
        // если система знает про сущ. запрашиваемого класса
        if (isset(self::$aliasList[$alias]))
            // поднимаем объект этого класса
            return new self::$aliasList[$alias]();
        
        // если система ничего не знает про существование
        // запрашиваемого класса, то вертаем эррор        
        self::getError('class ' . $alias . ' is not found');
    }
    
    /*
     * метод наполняет список именами классов инструментария
     * 
     * @access - public
     * 
     * @param string alias - псевдоним по которому будет доступен класс
     * @param string class - имя класса
     */
    
    public static function set($alias, $class)
    {
        self::$aliasList[$alias] = $class;
    }
    
    /*
     * метод вертает ошибку
     * 
     * @access - public
     * 
     * @param string message - сообщение ошибки
     */
    
    private static function getError($message)
    {
        // закрываем sql подключение
        sql::close();
        
        die('RS-MINI ERROR: ' . $message);
    }
}
