<?php

/*
 * @package RS-MINI
 * @copyright (c) 2015-2016 Alexey Glumov aka Rio-Shaman (support@rio-shaman.ru)
 * @license GNU General Public License version 2; see LICENSE.txt
 *
 */
 
namespace core;

if(!defined('RS-MINI')) die();

/*
 * room v 1.0
 * 
 * класс по работе с кабинетом системы
 *
 */

class room
{
    /*
     * инфа о кабинете
     *
     * @var    - array
     * @access - private
     */
    
    private static $room = NULL;
    
    /*
     * метод имитирует вызов методов получения
     * инфы о кабинете. к таким методам относятся:
     *  - ::getID()      - id кабинета
     *  - ::getName()    - имя кабинета
     *  - ::getFolder()  - физ папка кабинета
     *  - ::getSection() - секция (ядро или проект)
     *  - ::getPath()    - путь на сайте до кабинета
     *  - ::getAccess()  - доступ к кабинету
     * 
     * @access - public
     * @return - информация кабинета или NULL
     *
     * @param string name         - имя вызываемого метода
     * @param array  argumentList - список аргументов
     */
    
    public static function __callStatic($name, $argumentList)
    {
        // приводим к нижнему регистру
        $name     = mb_strtolower($name);
        // в $argument помещаем часть имени метода
        $argument = mb_substr($name, 3, mb_strlen($name)-1);
        
        // если вызываемый метод начинается не на get
        if (mb_substr($name, 0, 3) != 'get')
            // вертаем null
            return NULL;
            
        // вертаем информацию, если она имеется или null если ее нет
        return (isset(self::$room['room_' . $argument])) ? self::$room['room_' . $argument] : NULL;
    }
    
    /*
     * метод устанавливает кабинет
     * 
     * @access - public
     * @return - возвращает TRUE в случае успеха или FALSE в случае неудачи
     */
    
    public static function setRoom()
    {
        // определяем кабинет
        // получаем адрес страницы
        $uri = request::getHttpServer('REQUEST_URI')->toUri();
        // бьем строку на массив
        $uri = explode('/', $uri);
        // удаляем последний элемент массива, так как он всегда пустой
        unset($uri[(count($uri) - 1)]); 

        // смысл в том что нам необходимо отсортировать пути к кабинетам от большего
        // кол-ва узлов к меньшему.
        // например если у нас имеется два кабинете
        //      1. публичный /
        //      2. кабинет администратора /admin/
        // то видим что до публичного кабинета ОДИН узлел а до админского ДВА узла.
        // стало быть если мы сейчас на странице /admin/document-a/ то скрипт должен
        // понять что мы в админском кабинете, а не в публичном (в адресе к публичному
        // кабинету ведь тоже присутствует ПЕРВЫЙ узел). если не отсортировать по
        // кол-ву узлов, то скрипт может закончить обработку на публичном кабинете
        // не сверяясь с админским. если же первым делом скрипт будет проверять тот
        // кабинет в котором больше узлов, то ошибка выдать неверный кабинет упадет
        // до нуля
        
        // массив в который сложим кабинеты предварительно подсчитав кол-во узлов
        $roomArr   = array();
        // получаем список кабинетов
        if (!$roomList = self::getRoomList())
            return FALSE;
        
        // листаем кабинеты
        foreach ($roomList as $roomKey => $room) {
            // бьем путь до кабинета на массив
            $path = explode('/', $room['room_path']);
            
            // наполняем массив кабинетом с подсчетом количества узлов до него
            $roomArr[] = array(
                'count'    => count($path),
                'room'     => $roomList[$roomKey]
            );
        }
        
        // сортируем кабинеты по полю count от большего к меньшему
        usort($roomArr, array('\core\room', 'fSort'));

        // листаем кабинеты
        foreach ($roomArr as $roomKey => $room) {

            // делаем из пути массив
            $path = explode('/', $room['room']['room_path']);
            // удаляем последний элемент массива, так как он всегда пустой
            unset($path[(count($path) - 1)]);
            
            // вытаскиваем из пути к странице столько же элементов сколько до кабинета
            // массив проверки
            $checkPath = array();
            // листаем узлы вбитые в адресную строку
            foreach ($uri as $key => $alias)
                // если ключ (0,1,2,3...n) меньше кол-ву узлов в кабинете
                if ($key < count($path))
                    // заполняем стек узлом из адресной строки
                    $checkPath[] = $alias;
            
            // сравниваем пути
            if ($path == $checkPath) {
                // если они равны, значит кабинет мы нашли, запоминаем его
                self::$room = $roomArr[$roomKey]['room'];
                
                // назначаем константы для быстрого доступа
                // основываясь на имя секции
                define('IMG',           '/' . self::getSection() . '/img/'); // путь к папке img кабинета
                define('JS',            '/' . self::getSection() . '/js/');  // путь к папке js кабинета
                define('CSS',           '/' . self::getSection() . '/css/'); // путь к папке css кабинета
                
                // контент всегда лежит в проекте
                define('CONTENT',       DIR_PROJECT       . '/content/');    // полный путь к папке контента
                define('SHORT_CONTENT', SHORT_DIR_PROJECT . '/content/');    // короткий путь к папке контента
                
                break;
            }
        }

        return (is_null(self::$room)) ? FALSE : TRUE; 
    }

    /*
     * статичный пользовательский метод для сортировки массива
     * 
     * @access - public
     * 
     */
     
    public static function fSort($a, $b)
    {
        if ($a['count'] == $b['count'])
            return 0;

        return ($a['count'] > $b['count']) ? -1 : 1;
    }
    
    /*
     * метод получает список кабинетов
     * 
     * @access - public
     * @return - массив с кабинетами
     *
     */
    
    private static function getRoomList()
    {
        $sql = ''
            . ' SELECT'
                . ' r.room_id       AS room_id,'
                . ' r.room_name     AS room_name,'
                . ' r.room_folder   AS room_folder,'
                . ' r.room_section  AS room_section,'
                . ' r.room_path     AS room_path,'
                . ' r.room_access   AS room_access'
            . ' FROM'
                . ' rs_room         AS r'
        ;
        
        return sql::query($sql)->fetch();
    }
}
